%% http://lsr.di.unimi.it/LSR/Item?id=1090
%% Originally contributed by Aaron Hill:
%% https://lists.gnu.org/archive/html/lilypond-user/2019-03/msg00257.html

\version "2.18.0"

#(define (lyric-hyphen-text-stencil grob)
   "Draws a LyricHyphen using an arbitrary text markup."
   (define (span-point side common dir)
     (let ((iv
            ; compatible with 2.19 onwards -vv
            ; (ly:generic-bound-extent side common)))
            (ly:grob-robust-relative-extent side common X)))
       (if (interval-empty? iv)
           (ly:grob-relative-coordinate side common X)
           (interval-bound iv dir))))
   (define (get-text-stencil grob)
     (let* ((orig (ly:grob-original grob))
            (into (ly:spanner-broken-into orig)))
       (grob-interpret-markup
        (ly:spanner-bound (if (null? into) orig (first into)) LEFT)
        (ly:grob-property grob 'text "-"))))
   (let* ((left-bound (ly:spanner-bound grob LEFT))
          (right-bound (ly:spanner-bound grob RIGHT))
          (common (ly:grob-common-refpoint left-bound right-bound X))
          (left-span (span-point left-bound common RIGHT))
          (right-span (span-point right-bound common LEFT))
          (span-length (- right-span left-span))
          (padding (ly:grob-property grob 'padding 0.1))
          (minimum-length (ly:grob-property grob 'minimum-length 0.3))
          (usable-length (- span-length
                           (if (zero? (ly:item-break-dir left-bound)) padding 0)
                           (if (zero? (ly:item-break-dir right-bound)) padding 0))))
     (if (< usable-length minimum-length) '()
         (let* ((dash-sten (get-text-stencil grob))
                (dash-extent (ly:stencil-extent dash-sten X))
                (dash-length (min (interval-length dash-extent) usable-length))
                (scaled-sten (ly:stencil-scale
                              (ly:stencil-translate-axis
                               dash-sten (- (interval-start dash-extent)) X)
                              (/ dash-length (interval-length dash-extent)) 1))
                (dash-period (max (ly:grob-property grob 'dash-period 1.0) dash-length))
                ;; Aaron Hill's computation of the dash count:
                ; (dash-count (1+ (floor (/ (- usable-length dash-length) dash-period))))
                ;; Lily's internal computation of the dash count:
                (dash-count (max 1 (ceiling (- (/ usable-length dash-period) 0.5))))
                (extra (- usable-length dash-length (* (- dash-count 1) dash-period)))
                (offset (+ (- left-span (ly:grob-relative-coordinate grob common X))
                          (if (zero? (ly:item-break-dir left-bound)) padding 0)
                          (/ extra 2))))
           (apply ly:stencil-add (map (lambda (n)
                                        (ly:stencil-translate-axis scaled-sten (+ offset (* n dash-period)) X))
                                   (iota dash-count)))))))

\layout {
  \context {
    \Lyrics
    \override LyricHyphen.stencil = #lyric-hyphen-text-stencil
  }
}
